@@ -2,8 +2,8 @@
 #include "avc1394.h"
 
 /* FCP Register Space */
-#define FCP_COMMAND_ADDR 0xFFFFF0000B00
-#define FCP_RESPONSE_ADDR 0xFFFFF0000D00
+#define FCP_COMMAND_ADDR 0xFFFFF0000B00ULL
+#define FCP_RESPONSE_ADDR 0xFFFFF0000D00ULL
 
 #define MAX_RESPONSE_SIZE 512
 #define AVC1394_RETRY 2
