@@ -53,7 +53,8 @@ extern unsigned char g_fcp_response[];
 
 int avc1394_send_command(raw1394handle_t handle, nodeid_t node, quadlet_t command)
 {
-    quadlet_t cmd = htonl(command);
+    quadlet_t cmd;
+	cmd = htonl(command);
 
     return cooked1394_write(handle, 0xffc0 | node, FCP_COMMAND_ADDR, 4, &cmd);
 }
@@ -68,6 +69,7 @@ int avc1394_send_command_block(raw1394handle_t handle, nodeid_t node,
                             command_len*4, command);
 }
 
+
 /*
  * Send an AV/C request to a device, wait for the corresponding AV/C
  * response and return that. This version only uses quadlet transactions.
@@ -119,15 +121,20 @@ quadlet_t avc1394_transaction(raw1394handle_t handle, nodeid_t node,
 #ifdef DEBUG
 		if (response != 0)
 			fprintf(stderr, "avc1394_transaction: Got AVC response 0x%0x (%s)\n", response, decode_response(response));
+		else
+			fprintf(stderr, "avc1394_transaction: no response\n");
 #endif
+		
+		if (response != 0) {
+			stop_avc_response_handler(handle);
+			return response;
+		}
 
-		stop_avc_response_handler(handle);
-		return (response == 0 ? -1 : response);
-    } while (--retry >= 0);
+	} while (--retry >= 0);
 
     stop_avc_response_handler(handle);
 
-    return -1;
+	return (response == 0 ? -1 : response);
 }
 
 /*
@@ -157,6 +164,7 @@ quadlet_t *avc1394_transaction_block(raw1394handle_t handle, nodeid_t node,
 			struct timespec ts = {0, AVC1394_SLEEP};
             fprintf(stderr,"send oops\n");
             nanosleep(&ts, NULL);
+			continue;
         }
 
         if ( poll( &raw1394_poll, 1, AVC1394_POLL_TIMEOUT) > 0 ) {
@@ -190,8 +198,10 @@ quadlet_t *avc1394_transaction_block(raw1394handle_t handle, nodeid_t node,
         }
 #endif
 		
-		stop_avc_response_handler(handle);
-		return response;
+		if (response != 0) {
+			stop_avc_response_handler(handle);
+			return response;
+		}
     } while (--retry >= 0);
 	
     stop_avc_response_handler(handle);
@@ -395,3 +405,79 @@ quadlet_t *avc1394_unit_info(raw1394handle_t handle, nodeid_t node)
     return response;
 }
 
+/*************** TARGET *******************************************************/
+
+avc1394_command_handler_t g_command_handler = NULL;
+
+int target_fcp_handler( raw1394handle_t handle, nodeid_t nodeid, int response, 
+	size_t length, unsigned char *data )
+{
+	int result;
+	struct avc1394_command_response cmd_resp;
+	quadlet_t *r = (quadlet_t *) &cmd_resp;
+	
+	/* initialize the response from the request */
+	memset(&cmd_resp, 0, sizeof(cmd_resp));
+	memcpy(r, data, length);
+
+	if ( response != 0 )
+	{
+		return 0;
+	}
+	else
+	{
+#ifdef DEBUG
+	    {
+		int q, z;
+		z = (length % 4) ? length + (4 - (length % 4)) : length;
+		printf("----> ");
+		for (q = 0; q < z / 4; q++)
+		    printf("%08x ", r[q]);
+		printf("(length %d)\n", length);
+		printf("----> type=0x%02x subunit_type=0x%x subunit_id=0x%x opcode=0x%x operand0=0x%x\n",
+			cmd_resp.status, cmd_resp.subunit_type,
+			cmd_resp.subunit_id, cmd_resp.opcode, cmd_resp.operand[0]);
+	    }
+#endif
+
+		result = g_command_handler( &cmd_resp );
+		
+		if (result == 0) 
+			cmd_resp.status = AVC1394_RESP_NOT_IMPLEMENTED;
+		
+#ifdef DEBUG
+	    {
+		int q, z;
+		z = (length % 4) ? length + (4 - (length % 4)) : length;
+		printf("<---- ");
+		for (q = 0; q < z / 4; q++)
+		    printf("%08x ", r[q]);
+		printf("(length %d)\n", length);
+		printf("<---- type=0x%02x subunit_type=0x%x subunit_id=0x%x opcode=0x%x operand0=0x%x\n",
+			cmd_resp.status, cmd_resp.subunit_type,
+			cmd_resp.subunit_id, cmd_resp.opcode, cmd_resp.operand[0]);
+	    }
+#endif
+		
+		return cooked1394_write(handle, 0xffc0 | nodeid, FCP_RESPONSE_ADDR, length, r);
+	}
+}
+
+
+int
+avc1394_init_target( raw1394handle_t handle, avc1394_command_handler_t cmd_handler)
+{
+	if (cmd_handler == NULL)
+		return -1;
+	g_command_handler = cmd_handler;
+	if (raw1394_set_fcp_handler( handle, target_fcp_handler ) < 0)
+		return -1;
+	return raw1394_start_fcp_listen( handle );
+}
+
+
+int
+avc1394_close_target( raw1394handle_t handle )
+{
+	return raw1394_stop_fcp_listen(handle);
+}
