@@ -238,7 +238,10 @@ char *avc1394_vcr_decode_status(quadlet_t response)
     } else if (resp2 == AVC1394_VCR_RESPONSE_TRANSPORT_STATE_LOAD_MEDIUM) {
         return("Loading Medium");
     } else if (resp2 == AVC1394_VCR_RESPONSE_TRANSPORT_STATE_RECORD) {
-        return("Recording");
+        if (resp3 == AVC1394_VCR_OPERAND_RECORD_PAUSE)
+            return("Recording Paused");
+        else
+            return("Recording");
     } else if (resp2 == AVC1394_VCR_RESPONSE_TRANSPORT_STATE_PLAY) {
         if (resp3 >= AVC1394_VCR_OPERAND_PLAY_FAST_FORWARD_1
                 && resp3 <= AVC1394_VCR_OPERAND_PLAY_FASTEST_FORWARD) {
@@ -293,6 +296,30 @@ avc1394_vcr_get_timecode(raw1394handle_t handle, nodeid_t node)
     return output;
 }
 
+/* Get the time code on tape in format HH:MM:SS:FF */
+int
+avc1394_vcr_get_timecode2(raw1394handle_t handle, nodeid_t node, char *output)
+{
+    quadlet_t  request[2];
+    quadlet_t *response;
+        
+    request[0] = STATVCR0 | AVC1394_VCR_COMMAND_TIME_CODE | 
+        AVC1394_VCR_OPERAND_TIME_CODE_STATUS;
+    request[1] = 0xFFFFFFFF;
+    response = avc1394_transaction_block( handle, node, request, 2, AVC1394_RETRY);
+    if (response == NULL) 
+		return -1;
+
+    // consumer timecode format
+    sprintf(output, "%2.2x:%2.2x:%2.2x:%2.2x",
+        response[1] & 0x000000ff,
+        (response[1] >> 8) & 0x000000ff,
+        (response[1] >> 16) & 0x000000ff,
+        (response[1] >> 24) & 0x000000ff);
+    
+    return 0;
+}
+
 
 /* Go to the time code on tape in format HH:MM:SS:FF */
 void
@@ -315,4 +342,3 @@ avc1394_vcr_seek_timecode(raw1394handle_t handle, nodeid_t node, char *timecode)
     
     avc1394_send_command_block( handle, node, request, 2);
 }
-
