@@ -29,9 +29,11 @@
 
 // Load up the needed includes.
 #include <config.h>
-#include <librom1394/rom1394.h>
-#include <libavc1394/avc1394.h>
-#include <libavc1394/avc1394_vcr.h>
+
+#include "../librom1394/rom1394.h"
+#include "../libavc1394/avc1394.h"
+#include "../libavc1394/avc1394_vcr.h"
+
 #include <libraw1394/raw1394.h>
 #include <libraw1394/csr.h>
 #include <sys/types.h>
@@ -81,7 +83,7 @@ int main (int argc, char *argv[])
 	int verbose = 0;
     int i, j;
 	int speed;
-	char *timecode;
+	char timecode[12];
 
 	if (argc < 2)
 	{
@@ -118,8 +120,7 @@ int main (int argc, char *argv[])
     	if (rom1394_get_directory(handle, i, &rom_dir) < 0)
     	{
     	    fprintf(stderr,"error reading config rom directory for node %d\n", i);
-            raw1394_destroy_handle(handle);
-    	    exit(1);
+    	    continue;
         }
 
 		for (j = 1; j < argc; ++j) {
@@ -186,10 +187,10 @@ int main (int argc, char *argv[])
     		printf( "%s\n", avc1394_vcr_decode_status(avc1394_vcr_status(handle, device)));
 	    
 	    } else if (strcmp("timecode", argv[i]) == 0) {
-	        if ( (timecode = avc1394_vcr_get_timecode(handle, device)) != NULL) {
+	        if (avc1394_vcr_get_timecode2(handle, device, timecode) == 0)
         		printf( "%s\n", timecode);
-        		free(timecode);
-    		}
+			else
+				printf( "--:--:--:--\n");
 	    
 	    } else if (strcmp("seek", argv[i]) == 0) {
 	        avc1394_vcr_seek_timecode(handle, device, argv[i+1]);
@@ -251,4 +252,3 @@ int main (int argc, char *argv[])
     raw1394_destroy_handle(handle);
 	return 0;
 }
-
